@@ -1,7 +1,9 @@
-from autoarray import conf
+from autoconf import conf
+
+dir(conf)
 from autoarray import exc
 from autoarray import masked
-from autoarray import plotters as plot
+from autoarray import plot
 from autoarray import simulator
 from autoarray import util
 from autoarray.dataset import data_converter
@@ -20,9 +22,9 @@ from autoarray.structures.grids import (
     GridIrregular as grid_irregular,
     GridRectangular as grid_rectangular,
     GridVoronoi as grid_voronoi,
-    Positions as positions,
+    Coordinates as coordinates,
 )
 from autoarray.structures.kernel import Kernel as kernel
 from autoarray.structures.visibilities import Visibilities as visibilities
 
-__version__ = '0.3.4.2'
+__version__ = '0.7.10'
