@@ -1,14 +1,13 @@
 import logging
 
+import numpy as np
 from scipy.stats import norm
 
-import numpy as np
-
 from autoarray import exc
-from autoarray.mask import mask as msk
-from autoarray.structures import kernel, arrays
-from autoarray.masked import masked_structures
 from autoarray.dataset import abstract_dataset, data_converter
+from autoarray.mask import mask as msk
+from autoarray.masked import masked_structures
+from autoarray.structures import kernel, arrays
 
 logger = logging.getLogger(__name__)
 
@@ -403,11 +402,15 @@ class Imaging(AbstractImagingDataSet):
         )
 
         self.psf = psf
-        self.name = name
+        self.__name = name
         self.background_noise_map = background_noise_map
         self.poisson_noise_map = poisson_noise_map
         self.background_sky_map = background_sky_map
 
+    @property
+    def name(self) -> str:
+        return self.__name
+
     @classmethod
     def manual(
         cls,
@@ -439,6 +442,7 @@ class Imaging(AbstractImagingDataSet):
         noise_map_path=None,
         noise_map_hdu=0,
         noise_map_from_image_and_background_noise_map=False,
+        noise_map_non_constant=False,
         psf_path=None,
         psf_hdu=0,
         resized_psf_shape=None,
@@ -615,6 +619,12 @@ class Imaging(AbstractImagingDataSet):
             convert_from_adus=convert_from_adus,
         )
 
+        if noise_map_non_constant:
+            if np.allclose(noise_map, noise_map[0] * np.ones(shape=noise_map.shape)):
+                noise_map = noise_map + (
+                    0.001 * noise_map[0] * np.random.uniform(size=noise_map.shape_1d)
+                )
+
         if psf_path is not None:
 
             psf = kernel.Kernel.from_fits(
