@@ -29,7 +29,7 @@ def fit(masked_dataset, model_data, inversion=None):
         )
 
 
-class DatasetFit(object):
+class DatasetFit:
 
     # noinspection PyUnresolvedReferences
     def __init__(self, mask, data, noise_map, model_data, inversion=None):
@@ -140,6 +140,13 @@ class DatasetFit(object):
         else:
             return self.evidence
 
+    @property
+    def total_inversions(self):
+        if self.inversion is None:
+            return 0
+        else:
+            return 1
+
 
 class ImagingFit(DatasetFit):
     def __init__(self, mask, image, noise_map, model_image, inversion=None):
