@@ -1,9 +1,11 @@
 import numpy as np
 
+from autoconf import conf
 from autoarray import exc
-from autoarray.structures import visibilities as vis
+from autoarray.structures import arrays, grids, visibilities as vis
 from autoarray.masked import masked_dataset as md
 from autoarray.util import inversion_util
+from scipy.interpolate import griddata
 
 
 def inversion(masked_dataset, mapper, regularization):
@@ -29,7 +31,7 @@ def inversion(masked_dataset, mapper, regularization):
         )
 
 
-class Inversion(object):
+class Inversion:
     def __init__(
         self,
         noise_map,
@@ -47,6 +49,67 @@ class Inversion(object):
         self.curvature_reg_matrix = curvature_reg_matrix
         self.reconstruction = reconstruction
 
+    def interpolated_reconstruction_from_shape_2d(self, shape_2d=None):
+        return self.interpolated_values_from_shape_2d(
+            values=self.reconstruction, shape_2d=shape_2d
+        )
+
+    def interpolated_errors_from_shape_2d(self, shape_2d=None):
+        return self.interpolated_values_from_shape_2d(
+            values=self.errors, shape_2d=shape_2d
+        )
+
+    def interpolated_values_from_shape_2d(self, values, shape_2d=None):
+
+        if shape_2d is not None:
+
+            grid = grids.Grid.bounding_box(
+                bounding_box=self.mapper.pixelization_grid.extent,
+                shape_2d=shape_2d,
+                buffer_around_corners=False,
+            )
+
+        elif (
+            conf.instance.general.get("inversion", "interpolated_grid_shape", str)
+            in "image_grid"
+        ):
+
+            grid = self.mapper.grid
+
+        elif (
+            conf.instance.general.get("inversion", "interpolated_grid_shape", str)
+            in "source_grid"
+        ):
+
+            dimension = int(np.sqrt(self.mapper.pixels))
+            shape_2d = (dimension, dimension)
+
+            grid = grids.Grid.bounding_box(
+                bounding_box=self.mapper.pixelization_grid.extent,
+                shape_2d=shape_2d,
+                buffer_around_corners=False,
+            )
+
+        else:
+
+            raise exc.InversionException(
+                "In the genenal.ini config file a valid option was not found for the"
+                "interpolated_grid_shape. Must be {image_grid, source_grid}"
+            )
+
+        interpolated_reconstruction = griddata(
+            points=self.mapper.pixelization_grid,
+            values=values,
+            xi=grid.in_2d_binned,
+            method="linear",
+        )
+
+        interpolated_reconstruction[np.isnan(interpolated_reconstruction)] = 0.0
+
+        return arrays.Array.manual_2d(
+            array=interpolated_reconstruction, pixel_scales=grid.pixel_scales
+        )
+
     @property
     def errors_with_covariance(self):
         return np.linalg.inv(self.curvature_reg_matrix)
@@ -227,7 +290,7 @@ class InversionImaging(Inversion):
     def normalized_residual_map(self):
         return inversion_util.inversion_normalized_residual_map_from_pixelization_values_and_reconstructed_data_1d(
             pixelization_values=self.reconstruction,
-            mapped_reconstructed_data=self.image,
+            data=self.image,
             noise_map_1d=self.noise_map,
             mask_1d_index_for_sub_mask_1d_index=self.mapper.grid.mask.regions._mask_1d_index_for_sub_mask_1d_index,
             all_sub_mask_1d_indexes_for_pixelization_1d_index=self.mapper.all_sub_mask_1d_indexes_for_pixelization_1d_index,
@@ -237,7 +300,7 @@ class InversionImaging(Inversion):
     def chi_squared_map(self):
         return inversion_util.inversion_chi_squared_map_from_pixelization_values_and_reconstructed_data_1d(
             pixelization_values=self.reconstruction,
-            mapped_reconstructed_data=self.image,
+            data=self.image,
             noise_map_1d=self.noise_map,
             mask_1d_index_for_sub_mask_1d_index=self.mapper.grid.mask.regions._mask_1d_index_for_sub_mask_1d_index,
             all_sub_mask_1d_indexes_for_pixelization_1d_index=self.mapper.all_sub_mask_1d_indexes_for_pixelization_1d_index,
