@@ -4,10 +4,12 @@ import scipy.spatial
 import scipy.spatial.qhull as qhull
 from functools import wraps
 from sklearn.cluster import KMeans
+import os
 
 from autoarray import decorator_util
 from autoarray import exc
 from autoarray.structures import abstract_structure
+from autoarray.masked import masked_structures
 from autoarray.mask import mask as msk
 from autoarray.util import (
     sparse_util,
@@ -19,7 +21,7 @@ from autoarray.util import (
 
 
 class AbstractGrid(abstract_structure.AbstractStructure):
-    def __new__(cls, grid_1d, mask, binned=None, *args, **kwargs):
+    def __new__(cls, grid, mask, store_in_1d=True, binned=None, *args, **kwargs):
         """A grid of coordinates, where each entry corresponds to the (y,x) coordinates at the centre of an \
         unmasked pixel. The positive y-axis is upwards and poitive x-axis to the right.
 
@@ -131,7 +133,9 @@ class AbstractGrid(abstract_structure.AbstractStructure):
                  grid[8] = [0.25, -0.25]
 
         """
-        obj = super(AbstractGrid, cls).__new__(cls=cls, structure_1d=grid_1d, mask=mask)
+        obj = super(AbstractGrid, cls).__new__(
+            cls=cls, structure=grid, mask=mask, store_in_1d=store_in_1d
+        )
         obj.interpolator = None
         obj.binned = None
         return obj
@@ -169,17 +173,59 @@ class AbstractGrid(abstract_structure.AbstractStructure):
             setattr(self, key, value)
         super(AbstractGrid, self).__setstate__(state[0:-1])
 
+    @property
+    def in_1d(self):
+        if self.store_in_1d:
+            return self
+        else:
+            return self.mask.mapping.grid_stored_1d_from_sub_grid_2d(sub_grid_2d=self)
+
     @property
     def in_2d(self):
-        return self.mask.mapping.sub_grid_2d_from_sub_grid_1d(sub_grid_1d=self)
+        if self.store_in_1d:
+            return self.mask.mapping.grid_stored_2d_from_sub_grid_1d(sub_grid_1d=self)
+        else:
+            return self
 
     @property
     def in_1d_binned(self):
-        return self.mask.mapping.grid_binned_from_sub_grid_1d(sub_grid_1d=self)
+        if self.store_in_1d:
+            return self.mask.mapping.grid_stored_1d_binned_from_sub_grid_1d(
+                sub_grid_1d=self
+            )
+        else:
+            sub_grid_1d = self.mask.mapping.grid_stored_1d_from_sub_grid_2d(
+                sub_grid_2d=self
+            )
+            return self.mask.mapping.grid_stored_1d_binned_from_sub_grid_1d(
+                sub_grid_1d=sub_grid_1d
+            )
 
     @property
     def in_2d_binned(self):
-        return self.mask.mapping.grid_2d_binned_from_sub_grid_1d(sub_grid_1d=self)
+        if self.store_in_1d:
+            return self.mask.mapping.grid_stored_2d_binned_from_sub_grid_1d(
+                sub_grid_1d=self
+            )
+        else:
+            sub_grid_1d = self.mask.mapping.grid_stored_1d_from_sub_grid_2d(
+                sub_grid_2d=self
+            )
+            return self.mask.mapping.grid_stored_2d_binned_from_sub_grid_1d(
+                sub_grid_1d=sub_grid_1d
+            )
+
+    def squared_distances_from_coordinate(self, coordinate=(0.0, 0.0)):
+        squared_distances = np.square(self[:, 0] - coordinate[0]) + np.square(
+            self[:, 1] - coordinate[1]
+        )
+        return masked_structures.MaskedArray(array=squared_distances, mask=self.mask)
+
+    def distances_from_coordinate(self, coordinate=(0.0, 0.0)):
+        distances = np.sqrt(
+            self.squared_distances_from_coordinate(coordinate=coordinate)
+        )
+        return masked_structures.MaskedArray(array=distances, mask=self.mask)
 
     def blurring_grid_from_kernel_shape(self, kernel_shape_2d):
 
@@ -194,7 +240,9 @@ class AbstractGrid(abstract_structure.AbstractStructure):
             origin=blurring_mask.origin,
         )
 
-        return blurring_mask.mapping.grid_from_grid_1d(grid_1d=blurring_grid_1d)
+        return blurring_mask.mapping.grid_stored_1d_from_grid_1d(
+            grid_1d=blurring_grid_1d
+        )
 
     def new_grid_with_binned_grid(self, binned_grid):
         # noinspection PyAttributeOutsideInit
@@ -214,6 +262,14 @@ class AbstractGrid(abstract_structure.AbstractStructure):
         )
         return self
 
+    @property
+    def in_1d_flipped(self):
+        return np.fliplr(self)
+
+    @property
+    def in_2d_flipped(self):
+        return np.stack((self.in_2d[:, :, 1], self.in_2d[:, :, 0]), axis=-1)
+
     @property
     @array_util.Memoizer()
     def in_radians(self):
@@ -345,7 +401,7 @@ class AbstractGrid(abstract_structure.AbstractStructure):
             origin=padded_mask.origin,
         )
 
-        padded_sub_grid = padded_mask.mapping.grid_from_sub_grid_1d(
+        padded_sub_grid = padded_mask.mapping.grid_stored_1d_from_sub_grid_1d(
             sub_grid_1d=padded_grid_1d
         )
 
@@ -374,7 +430,7 @@ class AbstractGrid(abstract_structure.AbstractStructure):
         """
 
         return Grid(
-            grid_1d=self.relocated_grid_from_grid_jit(
+            grid=self.relocated_grid_from_grid_jit(
                 grid=grid, border_grid=self.sub_border_grid
             ),
             mask=grid.mask,
@@ -398,16 +454,44 @@ class AbstractGrid(abstract_structure.AbstractStructure):
             grid=self.relocated_grid_from_grid_jit(
                 grid=pixelization_grid, border_grid=self.sub_border_grid
             ),
-            nearest_irregular_1d_index_for_mask_1d_index=pixelization_grid.nearest_irregular_1d_index_for_mask_1d_index,
+            nearest_pixelization_1d_index_for_mask_1d_index=pixelization_grid.nearest_pixelization_1d_index_for_mask_1d_index,
+        )
+
+    def output_to_fits(self, file_path, overwrite=False):
+
+        array_util.numpy_array_1d_to_fits(
+            array_2d=self.in_2d, file_path=file_path, overwrite=overwrite
         )
 
 
 class Grid(AbstractGrid):
     @classmethod
-    def from_sub_grid_1d_shape_2d_pixel_scales_and_sub_size(
-        cls, sub_grid_1d, shape_2d, pixel_scales, sub_size, origin=(0.0, 0.0)
+    def manual_1d(
+        cls,
+        grid,
+        shape_2d,
+        pixel_scales,
+        sub_size=1,
+        origin=(0.0, 0.0),
+        store_in_1d=True,
     ):
 
+        if type(grid) is list:
+            grid = np.asarray(grid)
+
+        if type(pixel_scales) is float:
+            pixel_scales = (pixel_scales, pixel_scales)
+
+        if grid.shape[-1] != 2:
+            raise exc.GridException(
+                "The final dimension of the input grid is not equal to 2 (e.g. the (y,x) coordinates)"
+            )
+
+        if 2 < len(grid.shape) > 3:
+            raise exc.GridException(
+                "The dimensions of the input grid array is not 2 or 3"
+            )
+
         mask = msk.Mask.unmasked(
             shape_2d=shape_2d,
             pixel_scales=pixel_scales,
@@ -415,79 +499,94 @@ class Grid(AbstractGrid):
             origin=origin,
         )
 
-        return mask.mapping.grid_from_sub_grid_1d(sub_grid_1d=sub_grid_1d)
+        if store_in_1d:
+            return mask.mapping.grid_stored_1d_from_sub_grid_1d(sub_grid_1d=grid)
+        else:
+            return mask.mapping.grid_stored_2d_from_sub_grid_1d(sub_grid_1d=grid)
 
     @classmethod
-    def from_sub_grid_2d_pixel_scales_and_sub_size(
-        cls, sub_grid_2d, pixel_scales, sub_size, origin=(0.0, 0.0)
+    def manual_2d(
+        cls, grid, pixel_scales, sub_size=1, origin=(0.0, 0.0), store_in_1d=True
     ):
 
-        shape = (
-            int(sub_grid_2d.shape[0] / sub_size),
-            int(sub_grid_2d.shape[1] / sub_size),
-        )
+        if type(grid) is list:
+            grid = np.asarray(grid)
+
+        if type(pixel_scales) is float:
+            pixel_scales = (pixel_scales, pixel_scales)
+
+        if grid.shape[-1] != 2:
+            raise exc.GridException(
+                "The final dimension of the input grid is not equal to 2 (e.g. the (y,x) coordinates)"
+            )
+
+        if 2 < len(grid.shape) > 3:
+            raise exc.GridException(
+                "The dimensions of the input grid array is not 2 or 3"
+            )
+
+        shape = (int(grid.shape[0] / sub_size), int(grid.shape[1] / sub_size))
 
         mask = msk.Mask.unmasked(
             shape_2d=shape, pixel_scales=pixel_scales, sub_size=sub_size, origin=origin
         )
 
-        return mask.mapping.grid_from_sub_grid_2d(sub_grid_2d=sub_grid_2d)
+        if store_in_1d:
+            return mask.mapping.grid_stored_1d_from_sub_grid_2d(sub_grid_2d=grid)
+        else:
+            sub_grid_1d = mask.mapping.grid_stored_1d_from_sub_grid_2d(sub_grid_2d=grid)
+            return mask.mapping.grid_stored_2d_from_sub_grid_1d(sub_grid_1d=sub_grid_1d)
 
     @classmethod
-    def manual_1d(cls, grid, shape_2d, pixel_scales, sub_size=1, origin=(0.0, 0.0)):
+    def manual_yx_1d(
+        cls,
+        y,
+        x,
+        shape_2d,
+        pixel_scales,
+        sub_size=1,
+        origin=(0.0, 0.0),
+        store_in_1d=True,
+    ):
 
-        if type(grid) is list:
-            grid = np.asarray(grid)
+        if type(y) is list:
+            y = np.asarray(y)
 
-        if type(pixel_scales) is float:
-            pixel_scales = (pixel_scales, pixel_scales)
+        if type(x) is list:
+            x = np.asarray(x)
 
-        if grid.shape[-1] != 2:
-            raise exc.GridException(
-                "The final dimension of the input grid is not equal to 2 (e.g. the (y,x) coordinates)"
-            )
-
-        if 2 < len(grid.shape) > 3:
-            raise exc.GridException(
-                "The dimensions of the input grid array is not 2 or 3"
-            )
-
-        return Grid.from_sub_grid_1d_shape_2d_pixel_scales_and_sub_size(
-            sub_grid_1d=grid,
+        return cls.manual_1d(
+            grid=np.stack((y, x), axis=-1),
             shape_2d=shape_2d,
             pixel_scales=pixel_scales,
             sub_size=sub_size,
             origin=origin,
+            store_in_1d=store_in_1d,
         )
 
     @classmethod
-    def manual_2d(cls, grid, pixel_scales, sub_size=1, origin=(0.0, 0.0)):
+    def manual_yx_2d(
+        cls, y, x, pixel_scales, sub_size=1, origin=(0.0, 0.0), store_in_1d=True
+    ):
 
-        if type(grid) is list:
-            grid = np.asarray(grid)
+        if type(y) is list:
+            y = np.asarray(y)
 
-        if type(pixel_scales) is float:
-            pixel_scales = (pixel_scales, pixel_scales)
+        if type(x) is list:
+            x = np.asarray(x)
 
-        if grid.shape[-1] != 2:
-            raise exc.GridException(
-                "The final dimension of the input grid is not equal to 2 (e.g. the (y,x) coordinates)"
-            )
-
-        if 2 < len(grid.shape) > 3:
-            raise exc.GridException(
-                "The dimensions of the input grid array is not 2 or 3"
-            )
-
-        return Grid.from_sub_grid_2d_pixel_scales_and_sub_size(
-            sub_grid_2d=grid,
+        return cls.manual_2d(
+            grid=np.stack((y, x), axis=-1),
             pixel_scales=pixel_scales,
             sub_size=sub_size,
             origin=origin,
+            store_in_1d=store_in_1d,
         )
 
     @classmethod
-    def uniform(cls, shape_2d, pixel_scales, sub_size=1, origin=(0.0, 0.0)):
+    def uniform(
+        cls, shape_2d, pixel_scales, sub_size=1, origin=(0.0, 0.0), store_in_1d=True
+    ):
 
         if type(pixel_scales) is float:
             pixel_scales = (pixel_scales, pixel_scales)
@@ -505,17 +604,34 @@ class Grid(AbstractGrid):
             pixel_scales=pixel_scales,
             sub_size=sub_size,
             origin=origin,
+            store_in_1d=store_in_1d,
         )
 
     @classmethod
-    def bounding_box(cls, bounding_box, shape_2d, sub_size=1):
+    def bounding_box(
+        cls,
+        bounding_box,
+        shape_2d,
+        sub_size=1,
+        buffer_around_corners=False,
+        store_in_1d=True,
+    ):
 
-        y_max, y_min, x_max, x_min = bounding_box
+        y_min, y_max, x_min, x_max = bounding_box
 
-        pixel_scales = (
-            (y_max - y_min) / (shape_2d[0] - 1),
-            (x_max - x_min) / (shape_2d[1] - 1),
-        )
+        if not buffer_around_corners:
+
+            pixel_scales = (
+                (y_max - y_min) / (shape_2d[0]),
+                (x_max - x_min) / (shape_2d[1]),
+            )
+
+        else:
+
+            pixel_scales = (
+                (y_max - y_min) / (shape_2d[0] - 1),
+                (x_max - x_min) / (shape_2d[1] - 1),
+            )
         origin = ((y_max + y_min) / 2.0, (x_max + x_min) / 2.0)
 
         return cls.uniform(
@@ -523,10 +639,11 @@ class Grid(AbstractGrid):
             pixel_scales=pixel_scales,
             sub_size=sub_size,
             origin=origin,
+            store_in_1d=store_in_1d,
         )
 
     @classmethod
-    def from_mask(cls, mask):
+    def from_mask(cls, mask, store_in_1d=True):
         """Setup a sub-grid of the unmasked pixels, using a mask and a specified sub-grid size. The center of \
         every unmasked pixel's sub-pixels give the grid's (y,x) scaled coordinates.
 
@@ -545,10 +662,15 @@ class Grid(AbstractGrid):
             origin=mask.origin,
         )
 
-        return mask.mapping.grid_from_sub_grid_1d(sub_grid_1d=sub_grid_1d)
+        if store_in_1d:
+            return mask.mapping.grid_stored_1d_from_sub_grid_1d(sub_grid_1d=sub_grid_1d)
+        else:
+            return mask.mapping.grid_stored_2d_from_sub_grid_1d(sub_grid_1d=sub_grid_1d)
 
     @classmethod
-    def blurring_grid_from_mask_and_kernel_shape(cls, mask, kernel_shape_2d):
+    def blurring_grid_from_mask_and_kernel_shape(
+        cls, mask, kernel_shape_2d, store_in_1d=True
+    ):
         """Setup a blurring-grid from a mask, where a blurring grid consists of all pixels that are masked, but they \
         are close enough to the unmasked pixels that a fraction of their light will be blurred into those pixels \
         via PSF convolution. For example, if our mask is as follows:
@@ -618,12 +740,19 @@ class Grid(AbstractGrid):
             origin=blurring_mask.origin,
         )
 
-        return blurring_mask.mapping.grid_from_grid_1d(grid_1d=blurring_grid_1d)
+        if store_in_1d:
+            return blurring_mask.mapping.grid_stored_1d_from_grid_1d(
+                grid_1d=blurring_grid_1d
+            )
+        else:
+            return blurring_mask.mapping.grid_stored_2d_from_grid_1d(
+                grid_1d=blurring_grid_1d
+            )
 
 
 class GridIrregular(np.ndarray):
     def __new__(
-        cls, grid, nearest_irregular_1d_index_for_mask_1d_index=None, *args, **kwargs
+        cls, grid, nearest_pixelization_1d_index_for_mask_1d_index=None, *args, **kwargs
     ):
         """A pixelization-grid of (y,x) coordinates which are used to form the pixel centres of adaptive pixelizations in the \
         *pixelizations* module.
@@ -640,22 +769,52 @@ class GridIrregular(np.ndarray):
         pix_grid : ndarray
             The grid of (y,x) scaled coordinates of every image-plane pixelization grid used for adaptive source \
             -plane pixelizations.
-        nearest_irregular_1d_index_for_mask_1d_index : ndarray
+        nearest_pixelization_1d_index_for_mask_1d_index : ndarray
             A 1D array that maps every grid pixel to its nearest pixelization-grid pixel.
         """
         if type(grid) is list:
             grid = np.asarray(grid)
         obj = grid.view(cls)
-        obj.nearest_irregular_1d_index_for_mask_1d_index = (
-            nearest_irregular_1d_index_for_mask_1d_index
+        obj.nearest_pixelization_1d_index_for_mask_1d_index = (
+            nearest_pixelization_1d_index_for_mask_1d_index
         )
         obj.interpolator = None
         return obj
 
+    def __array_finalize__(self, obj):
+        if hasattr(obj, "nearest_pixelization_1d_index_for_mask_1d_index"):
+            self.nearest_pixelization_1d_index_for_mask_1d_index = (
+                obj.nearest_pixelization_1d_index_for_mask_1d_index
+            )
+        if hasattr(obj, "interpolator"):
+            self.interpolator = obj.interpolator
+
+    def __reduce__(self):
+        # Get the parent's __reduce__ tuple
+        pickled_state = super(GridIrregular, self).__reduce__()
+        # Create our own tuple to pass to __setstate__
+        class_dict = {}
+        for key, value in self.__dict__.items():
+            class_dict[key] = value
+        new_state = pickled_state[2] + (class_dict,)
+        # Return a tuple that replaces the parent's __setstate__ tuple with our own
+        return pickled_state[0], pickled_state[1], new_state
+
+    # noinspection PyMethodOverriding
+    def __setstate__(self, state):
+
+        for key, value in state[-1].items():
+            setattr(self, key, value)
+        super(GridIrregular, self).__setstate__(state[0:-1])
+
     @classmethod
     def manual_1d(cls, grid):
         return GridIrregular(grid=grid)
 
+    @classmethod
+    def manual_yx_1d(cls, y, x):
+        return GridIrregular(grid=np.stack((y, x), axis=-1))
+
     @classmethod
     def from_grid_and_unmasked_2d_grid_shape(cls, unmasked_sparse_shape, grid):
 
@@ -665,16 +824,20 @@ class GridIrregular(np.ndarray):
 
         return GridIrregular(
             grid=sparse_grid.sparse,
-            nearest_irregular_1d_index_for_mask_1d_index=sparse_grid.sparse_1d_index_for_mask_1d_index,
+            nearest_pixelization_1d_index_for_mask_1d_index=sparse_grid.sparse_1d_index_for_mask_1d_index,
         )
 
-    def __array_finalize__(self, obj):
-        if hasattr(obj, "nearest_irregular_1d_index_for_mask_1d_index"):
-            self.nearest_irregular_1d_index_for_mask_1d_index = (
-                obj.nearest_irregular_1d_index_for_mask_1d_index
-            )
-        if hasattr(obj, "interpolator"):
-            self.interpolator = obj.interpolator
+    def squared_distances_from_coordinate(self, coordinate=(0.0, 0.0)):
+        squared_distances = np.square(self[:, 0] - coordinate[0]) + np.square(
+            self[:, 1] - coordinate[1]
+        )
+        return masked_structures.MaskedArray(array=squared_distances, mask=self.mask)
+
+    def distances_from_coordinate(self, coordinate=(0.0, 0.0)):
+        distances = np.sqrt(
+            self.squared_distances_from_coordinate(coordinate=coordinate)
+        )
+        return masked_structures.MaskedArray(array=distances, mask=self.mask)
 
     @property
     def sub_shape_1d(self):
@@ -682,7 +845,7 @@ class GridIrregular(np.ndarray):
 
     @property
     def mask(self):
-        class IrregularMask(object):
+        class IrregularMask:
             def __init__(self, sub_shape_1d):
 
                 self.sub_shape_1d = sub_shape_1d
@@ -694,14 +857,14 @@ class GridIrregular(np.ndarray):
 
     @property
     def mapping(self):
-        class IrregularMapping(object):
+        class IrregularMapping:
             def __init__(self):
                 pass
 
-            def array_from_sub_array_1d(self, sub_array_1d):
+            def array_stored_1d_from_sub_array_1d(self, sub_array_1d):
                 return sub_array_1d
 
-            def grid_from_sub_grid_1d(self, sub_grid_1d):
+            def grid_stored_1d_from_sub_grid_1d(self, sub_grid_1d):
                 return GridIrregular(grid=sub_grid_1d)
 
         return IrregularMapping()
@@ -733,7 +896,7 @@ class GridIrregular(np.ndarray):
         )
 
 
-class SparseGrid(object):
+class SparseGrid:
     def __init__(self, sparse_grid, sparse_1d_index_for_mask_1d_index):
         """A sparse grid of coordinates, where each entry corresponds to the (y,x) coordinates at the centre of a \
         pixel on the sparse grid. To setup the sparse-grid, it is laid over a grid of unmasked pixels, such \
@@ -875,7 +1038,7 @@ class SparseGrid(object):
 
         try:
             kmeans = kmeans.fit(X=grid.in_1d_binned, sample_weight=weight_map)
-        except ValueError:
+        except ValueError or OverflowError:
             raise exc.InversionException()
 
         return SparseGrid(
@@ -889,9 +1052,7 @@ class SparseGrid(object):
 
 
 class GridRectangular(Grid):
-    def __new__(
-        cls, grid_1d, shape_2d, pixel_scales, origin=(0.0, 0.0), *args, **kwargs
-    ):
+    def __new__(cls, grid, shape_2d, pixel_scales, origin=(0.0, 0.0), *args, **kwargs):
         """A pixelization-grid of (y,x) coordinates which are used to form the pixel centres of adaptive pixelizations in the \
         *pixelizations* module.
 
@@ -907,7 +1068,7 @@ class GridRectangular(Grid):
         pix_grid : ndarray
             The grid of (y,x) scaled coordinates of every image-plane pixelization grid used for adaptive source \
             -plane pixelizations.
-        nearest_irregular_1d_index_for_mask_1d_index : ndarray
+        nearest_pixelization_1d_index_for_mask_1d_index : ndarray
             A 1D array that maps every grid pixel to its nearest pixelization-grid pixel.
         """
 
@@ -915,7 +1076,7 @@ class GridRectangular(Grid):
             shape_2d=shape_2d, pixel_scales=pixel_scales, sub_size=1, origin=origin
         )
 
-        obj = super(GridRectangular, cls).__new__(cls=cls, grid_1d=grid_1d, mask=mask)
+        obj = super(GridRectangular, cls).__new__(cls=cls, grid=grid, mask=mask)
         pixel_neighbors, pixel_neighbors_size = pixelization_util.rectangular_neighbors_from_shape(
             shape=shape_2d
         )
@@ -926,6 +1087,24 @@ class GridRectangular(Grid):
     def __array_finalize__(self, obj):
         pass
 
+    def __reduce__(self):
+        # Get the parent's __reduce__ tuple
+        pickled_state = super(GridRectangular, self).__reduce__()
+        # Create our own tuple to pass to __setstate__
+        class_dict = {}
+        for key, value in self.__dict__.items():
+            class_dict[key] = value
+        new_state = pickled_state[2] + (class_dict,)
+        # Return a tuple that replaces the parent's __setstate__ tuple with our own
+        return pickled_state[0], pickled_state[1], new_state
+
+    # noinspection PyMethodOverriding
+    def __setstate__(self, state):
+
+        for key, value in state[-1].items():
+            setattr(self, key, value)
+        super(GridRectangular, self).__setstate__(state[0:-1])
+
     @classmethod
     def overlay_grid(cls, shape_2d, grid, buffer=1e-8):
         """The geometry of a rectangular grid.
@@ -965,7 +1144,7 @@ class GridRectangular(Grid):
         )
 
         return GridRectangular(
-            grid_1d=grid_1d, shape_2d=shape_2d, pixel_scales=pixel_scales, origin=origin
+            grid=grid_1d, shape_2d=shape_2d, pixel_scales=pixel_scales, origin=origin
         )
 
     @property
@@ -1001,7 +1180,11 @@ class GridVoronoi(GridIrregular):
     """
 
     def __new__(
-        cls, grid_1d, nearest_irregular_1d_index_for_mask_1d_index=None, *args, **kwargs
+        cls,
+        grid_1d,
+        nearest_pixelization_1d_index_for_mask_1d_index=None,
+        *args,
+        **kwargs
     ):
         """A pixelization-grid of (y,x) coordinates which are used to form the pixel centres of adaptive pixelizations in the \
         *pixelizations* module.
@@ -1018,7 +1201,7 @@ class GridVoronoi(GridIrregular):
         pix_grid : ndarray
             The grid of (y,x) scaled coordinates of every image-plane pixelization grid used for adaptive source \
             -plane pixelizations.
-        nearest_irregular_1d_index_for_mask_1d_index : ndarray
+        nearest_pixelization_1d_index_for_mask_1d_index : ndarray
             A 1D array that maps every grid pixel to its nearest pixelization-grid pixel.
         """
 
@@ -1031,7 +1214,7 @@ class GridVoronoi(GridIrregular):
                 np.asarray([grid_1d[:, 1], grid_1d[:, 0]]).T,
                 qhull_options="Qbb Qc Qx Qm",
             )
-        except OverflowError or scipy.spatial.qhull.QhullError:
+        except ValueError or OverflowError or scipy.spatial.qhull.QhullError:
             raise exc.PixelizationException()
 
         pixel_neighbors, pixel_neighbors_size = pixelization_util.voronoi_neighbors_from_pixels_and_ridge_points(
@@ -1040,14 +1223,45 @@ class GridVoronoi(GridIrregular):
 
         obj.pixel_neighbors = pixel_neighbors.astype("int")
         obj.pixel_neighbors_size = pixel_neighbors_size.astype("int")
-        obj.nearest_irregular_1d_index_for_mask_1d_index = (
-            nearest_irregular_1d_index_for_mask_1d_index
+        obj.nearest_pixelization_1d_index_for_mask_1d_index = (
+            nearest_pixelization_1d_index_for_mask_1d_index
         )
 
         return obj
 
+    def __array_finalize__(self, obj):
 
-class Interpolator(object):
+        if hasattr(obj, "pixel_neighbors"):
+            self.pixel_neighbors = obj.pixel_neighbors
+
+        if hasattr(obj, "pixel_neighbors_size"):
+            self.pixel_neighbors_size = obj.pixel_neighbors_size
+
+        if hasattr(obj, "nearest_pixelization_1d_index_for_mask_1d_index"):
+            self.nearest_pixelization_1d_index_for_mask_1d_index = (
+                obj.nearest_pixelization_1d_index_for_mask_1d_index
+            )
+
+    def __reduce__(self):
+        # Get the parent's __reduce__ tuple
+        pickled_state = super(GridVoronoi, self).__reduce__()
+        # Create our own tuple to pass to __setstate__
+        class_dict = {}
+        for key, value in self.__dict__.items():
+            class_dict[key] = value
+        new_state = pickled_state[2] + (class_dict,)
+        # Return a tuple that replaces the parent's __setstate__ tuple with our own
+        return pickled_state[0], pickled_state[1], new_state
+
+    # noinspection PyMethodOverriding
+    def __setstate__(self, state):
+
+        for key, value in state[-1].items():
+            setattr(self, key, value)
+        super(GridVoronoi, self).__setstate__(state[0:-1])
+
+
+class Interpolator:
     def __init__(self, grid, interp_grid, pixel_scale_interpolation_grid):
         self.grid = grid
         self.interp_grid = interp_grid
@@ -1092,7 +1306,9 @@ class Interpolator(object):
 
         return Interpolator(
             grid=grid,
-            interp_grid=interp_mask.mapping.grid_from_grid_1d(grid_1d=interp_grid),
+            interp_grid=interp_mask.mapping.grid_stored_1d_from_grid_1d(
+                grid_1d=interp_grid
+            ),
             pixel_scale_interpolation_grid=pixel_scale_interpolation_grid,
         )
 
@@ -1100,74 +1316,222 @@ class Interpolator(object):
         return np.einsum("nj,nj->n", np.take(values, self.vtx), self.wts)
 
 
-class Positions(list):
-    def __init__(self, positions):
+class Coordinates(list):
+    def __init__(self, coordinates, mask=None):
 
-        positions = list(
-            map(
-                lambda position_set: GridIrregular.manual_1d(
-                    grid=np.asarray(position_set)
-                ),
-                positions,
-            )
-        )
+        super(Coordinates, self).__init__(coordinates)
 
-        super(Positions, self).__init__(positions)
+        self.mask = mask
 
     @classmethod
-    def from_file(cls, positions_path):
-        """Load the positions of an image.
+    def from_pixels_and_mask(cls, pixels, mask):
+        coordinates = []
+        for coordinate_set in pixels:
+            coordinates.append(
+                [
+                    mask.geometry.scaled_coordinates_from_pixel_coordinates(
+                        pixel_coordinates=coordinates
+                    )
+                    for coordinates in coordinate_set
+                ]
+            )
+        return cls(coordinates=coordinates, mask=mask)
 
-        Positions correspond to a set of pixels in the lensed source galaxy that are anticipated to come from the same \
+    @property
+    def in_1d(self):
+
+        total_coordinates = sum([len(coordinate_set) for coordinate_set in self])
+
+        grid_1d = np.zeros(shape=(total_coordinates, 2))
+
+        coordinate_index = 0
+
+        for coordinate_set in self:
+            for coordinate in coordinate_set:
+                grid_1d[coordinate_index, :] = coordinate
+                coordinate_index += 1
+
+        return GridIrregular.manual_1d(grid=grid_1d)
+
+    @property
+    def list_in_1d(self):
+
+        coordinates_list = []
+
+        for coordinate_set in self:
+            coordinates_list.append(GridIrregular(grid=coordinate_set))
+
+        return coordinates_list
+
+    def from_1d_coordinates(self, coordinates_1d):
+
+        coordinate_1d_index = 0
+
+        new_coordinates = []
+
+        for coordinate_set_index in range(len(self)):
+            new_coordinate_set_list = []
+            for coordinates_index in range(len(self[coordinate_set_index])):
+                new_coordinate_set_list.append(
+                    tuple(coordinates_1d[coordinate_1d_index, :])
+                )
+                coordinate_1d_index += 1
+            new_coordinates.append(new_coordinate_set_list)
+
+        return Coordinates(coordinates=new_coordinates, mask=self.mask)
+
+    def from_1d_values(self, values_1d):
+
+        values_1d_index = 0
+
+        new_values_list = []
+
+        for coordinate_set_index in range(len(self)):
+            new_values_set_list = []
+            for coordinates_index in range(len(self[coordinate_set_index])):
+                new_values_set_list.append(values_1d[values_1d_index])
+                values_1d_index += 1
+            new_values_list.append(new_values_set_list)
+
+        return new_values_list
+
+    @property
+    def scaled(self):
+        return self
+
+    @property
+    def pixels(self):
+        coordinates = []
+        for coordinate_set in self:
+            coordinates.append(
+                [
+                    self.mask.geometry.pixel_coordinates_from_scaled_coordinates(
+                        scaled_coordinates=coordinates
+                    )
+                    for coordinates in coordinate_set
+                ]
+            )
+        return self.__class__(coordinates=coordinates, mask=self.mask)
+
+    @classmethod
+    def from_file(cls, file_path):
+        """Load the coordinates of an image.
+
+        Coordinates correspond to a set of pixels in the lensed source galaxy that are anticipated to come from the same \
         multiply-imaged region of the source-plane. Mass models which do not trace the pixels within a threshold value of \
         one another are resampled during the non-linear search.
 
-        Positions are stored in a .dat file, where each line of the file gives a list of list of (y,x) positions which \
+        Coordinates are stored in a .dat file, where each line of the file gives a list of list of (y,x) coordinates which \
         correspond to the same region of the source-plane. Thus, multiple source-plane regions can be input over multiple \
-        lines of the same positions file.
+        lines of the same coordinates file.
 
         Parameters
         ----------
-        positions_path : str
-            The path to the positions .dat file containing the positions (e.g. '/path/to/positions.dat')
+        file_path : str
+            The path to the coordinates .dat file containing the coordinates (e.g. '/path/to/coordinates.dat')
         """
-        with open(positions_path) as f:
-            position_string = f.readlines()
+        with open(file_path) as f:
+            coordinate_string = f.readlines()
 
-        positions = []
+        coordinates = []
 
-        for line in position_string:
-            position_list = ast.literal_eval(line)
-            positions.append(position_list)
+        for line in coordinate_string:
+            coordinate_list = ast.literal_eval(line)
+            coordinates.append(coordinate_list)
 
-        return positions
+        return Coordinates(coordinates=coordinates)
 
-    def output_to_file(self, positions_path):
-        """Output the positions of an image to a positions.dat file.
+    def output_to_file(self, file_path, overwrite=False):
+        """Output the coordinates of an image to a coordinates.dat file.
 
-        Positions correspond to a set of pixels in the lensed source galaxy that are anticipated to come from the same \
+        Coordinates correspond to a set of pixels in the lensed source galaxy that are anticipated to come from the same \
         multiply-imaged region of the source-plane. Mass models which do not trace the pixels within a threshold value of \
         one another are resampled during the non-linear search.
 
-        Positions are stored in a .dat file, where each line of the file gives a list of list of (y,x) positions which \
+        Coordinates are stored in a .dat file, where each line of the file gives a list of list of (y,x) coordinates which \
         correspond to the same region of the source-plane. Thus, multiple source-plane regions can be input over multiple \
-        lines of the same positions file.
+        lines of the same coordinates file.
 
         Parameters
         ----------
-        positions : [[[]]]
-            The lists of positions (e.g. [[[1.0, 1.0], [2.0, 2.0]], [[3.0, 3.0], [4.0, 4.0]]])
-        positions_path : str
-            The path to the positions .dat file containing the positions (e.g. '/path/to/positions.dat')
+        coordinates : [[()]]
+            The lists of coordinates (e.g. [[(1.0, 1.0), (2.0, 2.0)], [(3.0, 3.0), (4.0, 4.0)]])
+        file_path : str
+            The path to the coordinates .dat file containing the coordinates (e.g. '/path/to/coordinates.dat')
         """
 
-        positions_out = list(
-            map(lambda position_set: np.ndarray.tolist(position_set), self)
-        )
+        if overwrite and os.path.exists(file_path):
+            os.remove(file_path)
+        elif not overwrite and os.path.exists(file_path):
+            raise FileExistsError(
+                "The file ",
+                file_path,
+                " already exists. Set overwrite=True to overwrite this" "file",
+            )
 
-        with open(positions_path, "w") as f:
-            for position in positions_out:
-                f.write("%s\n" % position)
+        with open(file_path, "w") as f:
+            for coordinate in self:
+                f.write("%s\n" % coordinate)
+
+
+def convert_coordinates_to_grid(func):
+    """ Checks whether any coordinates in the grid are radially near (0.0, 0.0), which can lead to numerical faults in \
+    the evaluation of a light or mass profiles. If any coordinates are radially within the the radial minimum \
+    threshold, their (y,x) coordinates are shifted to that value to ensure they are evaluated correctly.
+
+    By default this radial minimum is not used, and users should be certain they use a value that does not impact \
+    results.
+
+    Parameters
+    ----------
+    func : (profile, *args, **kwargs) -> Object
+        A function that takes a grid of coordinates which may have a singularity as (0.0, 0.0)
+
+    Returns
+    -------
+        A function that can except cartesian or transformed coordinates
+    """
+
+    @wraps(func)
+    def wrapper(profile, grid, *args, **kwargs):
+        """
+
+        Parameters
+        ----------
+        profile : SphericalProfile
+            The profiles that owns the function
+        grid : ndarray
+            PlaneCoordinates in either cartesian or profiles coordinate system
+        args
+        kwargs
+
+        Returns
+        -------
+            A value or coordinate in the same coordinate system as those passed in.
+        """
+
+        if isinstance(grid, Coordinates):
+            values_1d = func(profile, grid.in_1d, *args, **kwargs)
+            if isinstance(values_1d, np.ndarray):
+                if len(values_1d.shape) == 1:
+                    return grid.from_1d_values(values_1d=values_1d)
+                elif len(values_1d.shape) == 2:
+                    return grid.from_1d_coordinates(coordinates_1d=values_1d)
+            elif isinstance(values_1d, list):
+                if len(values_1d[0].shape) == 1:
+                    return [
+                        grid.from_1d_values(values_1d=value_1d)
+                        for value_1d in values_1d
+                    ]
+                elif len(values_1d[0].shape) == 2:
+                    return [
+                        grid.from_1d_coordinates(coordinates_1d=value_1d)
+                        for value_1d in values_1d
+                    ]
+        else:
+            return func(profile, grid, *args, **kwargs)
+
+    return wrapper
 
 
 def grid_interpolate(func):
