@@ -438,6 +438,66 @@ class TestEllipiticalAnnular:
         assert mask.geometry.mask_centre == (0.0, 0.0)
 
 
+class TestFromPixelCoordinates:
+    def test__mask_with_or_without_buffer__false_at_buffed_coordinates(self):
+
+        mask = aa.mask.from_pixel_coordinates(
+            shape_2d=(5, 5), pixel_coordinates=[[2, 2]], pixel_scales=1.0, buffer=0
+        )
+
+        assert (
+            mask
+            == np.array(
+                [
+                    [True, True, True, True, True],
+                    [True, True, True, True, True],
+                    [True, True, False, True, True],
+                    [True, True, True, True, True],
+                    [True, True, True, True, True],
+                ]
+            )
+        ).all()
+
+        mask = aa.mask.from_pixel_coordinates(
+            shape_2d=(5, 5), pixel_coordinates=[[2, 2]], pixel_scales=1.0, buffer=1
+        )
+
+        assert (
+            mask
+            == np.array(
+                [
+                    [True, True, True, True, True],
+                    [True, False, False, False, True],
+                    [True, False, False, False, True],
+                    [True, False, False, False, True],
+                    [True, True, True, True, True],
+                ]
+            )
+        ).all()
+
+        mask = aa.mask.from_pixel_coordinates(
+            shape_2d=(7, 7),
+            pixel_coordinates=[[2, 2], [5, 5]],
+            pixel_scales=1.0,
+            buffer=1,
+        )
+
+        assert (
+            mask
+            == np.array(
+                [
+                    [True, True, True, True, True, True, True],
+                    [True, False, False, False, True, True, True],
+                    [True, False, False, False, True, True, True],
+                    [True, False, False, False, True, True, True],
+                    [True, True, True, True, False, False, False],
+                    [True, True, True, True, False, False, False],
+                    [True, True, True, True, False, False, False],
+                ]
+            )
+        ).all()
+
+
 class TestFromAndToFits:
     def test__load_and_output_mask_to_fits(self):
 
